/*
Copyright 2004 Cesar Tardaguila & Javier Tardaguila

This file is part of Drops.

Drops is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

Srops is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with Drops; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

import net.designnation.data.*
import net.designnation.events.*
import net.designnation.behaviours.*
import net.designnation.Physics.*

import Code.Ents.*

class Code.World.DropsWorld extends EventSource
{
	private static var FRAME_RATE	: Number = 24;
	private static var INIT_TIME	: Number = 240;
	private static var INIT_MAXNUMBER: Number = 200;// 50 parece bien
	private static var DELTA_COEF	: Number = 0.9;
	private static var STAGE_SIZE	: Number = 400;

	private var base_MC	: MovieClip;
	private var stageMC	: MovieClip;
	
	private var BEngineVal	: BEngine;
	private var mySMachine	: SMachine;
	
	private var feedbackClickedFlag	: Boolean;
	private var newGameFlag			: Boolean;
	private var endOfGameFlag		: Boolean;
	private var victoryFlag			: Boolean;
	
	private var remainTime	: Number;
	private var numberDelta	: Number;
	private var actualIndex : Number;
	private var maxNumber	: Number;
	private var roundNumber	: Number;
	
	private var entsCollection	: HashMap;


	public function DropsWorld( baseMCParam: MovieClip )
	{
		this.base_MC = baseMCParam;
		
		this.entsCollection = new HashMap( );
		
		this.mySMachine		= new SMachine(  );
		
		this.BEngineVal		= new BEngine( );
		
		this.BEngineVal.registerSMachine( this.mySMachine );
		
	}
	
	public function initWorld( param: Object )
	{
		this.stageMC = param.baseline;
	
		this.initBehaviour( );
		
		var theClass: DropsWorld = this;
		
		this.base_MC.onEnterFrame = function( )
		{
			theClass.doProcess( );
		}
		
		this.mySMachine.startMachine( );
	}
	
	private function doProcess( )
	{
		this.BEngineVal.doProcess( );
	}
	
	private function initBehaviour( )
	{
		var initGame		: State= new State( "initGame",
								new CallbackDecl( this, "initGameAction" ) );
								
		var startGame		: State= new State( "startGame",
								new CallbackDecl( this, "startGameAction" ) );
								
		var createDrop		: State= new State( "createDrop",
								new CallbackDecl( this, "createDropAction" ) );
								
		var overDrops		: State = new State( "overDrops",
								new CallbackDecl( this, "overDropsAction" ) );								
								
		var endOfTime		: State= new State( "endOfTime",
								new CallbackDecl( this, "endOfTimeAction" ) );
								
		var defeat			: State= new State( "defeat",
								new CallbackDecl( this, "defeatAction" ) );
								
		var victory			: State= new State( "victory",
								new CallbackDecl( this, "victoryAction" ) );								
								
		var endOfGame		: State= new State( "endOfGame",
								new CallbackDecl( this, "endOfGameAction" ) );
								
								
								
		new Transition( "initGameToStartGame", initGame, startGame, 
			new CallbackDecl( this, "initGameToStartGameEval" ) );
		//----------------------------------------------------------	
		new Transition( "startGameToCreateDrop", startGame, createDrop, 
			new CallbackDecl( this, "startGameToCreateDropEval" ) );
		//----------------------------------------------------------	
		new Transition( "createDropToEndOfTime", createDrop, endOfTime, 
			new CallbackDecl( this, "createDropToEndOfTimeEval" ) );
			
		new Transition( "createDropToOverDrops", createDrop, overDrops, 
			new CallbackDecl( this, "createBubbleToOverDropsEval" ) );

		new Transition( "createDropToSelf", createDrop, createDrop, 
			new CallbackDecl( this, "createDropToSelfEval" ) );
			
		//----------------------------------------------------------	
		new Transition( "overDropsToDefeat", overDrops, defeat, 
			new CallbackDecl( this, "overDropsToDefeatEval" ) );
		//-----------------------------------------------------------			
		new Transition( "endOfTimeToVictory", endOfTime, victory, 
			new CallbackDecl( this, "endOfTimeToVictoryEval" ) );
			
		new Transition( "endOfTimeToStartGame", endOfTime, startGame, 
			new CallbackDecl( this, "endOfTimeToStartGameEval" ) );
			
		//-------------------------------------------------------------
		new Transition( "defeatToEndOfGame", defeat, endOfGame, 
			new CallbackDecl( this, "defeatToEndOfGameEval" ) );
			
		//-------------------------------------------------------------
		new Transition( "victoryToEndOfGame", victory, endOfGame, 
			new CallbackDecl( this, "victoryToEndOfGameEval" ) );			

								
		this.mySMachine.resetToInit( initGame );		
								
	}
	
	//States
	
	private function initGameAction( args: Object )
	{
		this.feedbackClickedFlag = false;
		this.numberDelta = 1;
		this.roundNumber = 0;
		
		fireEvent( "onStartGame", { timeline: this.base_MC } );
	}
	
	private function startGameAction( args: Object )
	{
		//trace( "startGameAction" );
		this.newGameFlag 	= false;
		this.endOfGameFlag 	= false;
		this.victoryFlag 	= false;
		
		this.remainTime		= INIT_TIME;
		this.maxNumber		= INIT_MAXNUMBER* this.numberDelta;
		
		this.actualIndex 	= 0;
		
		this.roundNumber++;
	}	
	
	private function createDropAction( args: Object )
	{
		var theDrop: Drop = new Drop( this.base_MC, "drop"+ this.actualIndex );
		
		theDrop.registerEvent( this, "onDropClicked" );
		
		var randomX: Number = Math.random( )* STAGE_SIZE;
		var randomY: Number = Math.random( )* STAGE_SIZE;
		
		var initArgs: Object = new Object( );
		initArgs.posIni = new Point( randomX, randomY );
		
		theDrop.initDrop( initArgs );
		
		this.entsCollection.setItem( "drop"+ this.actualIndex, theDrop );		

		if ( this.remainTime % FRAME_RATE == 0 )
		{
			var timeToShow: Number = Math.floor( this.remainTime / FRAME_RATE );
			
			fireEvent( "onTimeChange", { timeToShow: timeToShow } );
		}
		
		this.actualIndex++;
		this.remainTime--;
	}
	
	private function endOfTimeAction( args: Object )
	{
		//trace( "-----endOfTimeAction" );
		
		var it: IIterator= this.entsCollection.iterator(  );
		
		while( it.hasNext(  ) )
		{
			var bubble = it.next( ).secondVal;
		
			bubble.notClickeable( );			
		}		
		
		fireEvent( "onEndOfTime", { timeline: this.base_MC } );
	}
	
	private function overDropsAction( args: Object )
	{
		//trace( "-----overDropsAction" );
	}
	
	private function defeatAction( param: Object )
	{
		//trace( "-----defeatAction" );

		var it: IIterator= this.entsCollection.iterator(  );
		
		while( it.hasNext(  ) )
		{
			var bubble = it.next( ).secondVal;
		
			bubble.notClickeable( );			
		}				
		
		this.victoryFlag = false;
	}
	
	private function victoryAction( param: Object )
	{
		//trace ( "----victoryAction" );
		
		var it: IIterator= this.entsCollection.iterator(  );
		
		while( it.hasNext(  ) )
		{
			var bubble = it.next( ).secondVal;
		
			bubble.notClickeable( );			
		}				
		
		this.victoryFlag = true;
	}
	
	private function endOfGameAction( param: Object )
	{
		fireEvent( "onEndOfGame", { victoryFlag: this.victoryFlag, timeline: this.base_MC } );
	}
	
	//Transitions
	
	private function initGameToStartGameEval( param: Object ): Boolean
	{
		return this.feedbackClickedFlag;
	}
	
	private function startGameToCreateDropEval( param: Object ): Boolean
	{
		return true;
	}
	
	private function createBubbleToOverDropsEval( param: Object ): Boolean
	{
		return ( this.actualIndex>= this.maxNumber );
	}
	
	private function createDropToEndOfTimeEval( param: Object ): Boolean
	{
		return ( this.remainTime<= 0 );
	}
	
	private function createDropToSelfEval( param: Object ): Boolean
	{
		this.remainTime--;
		
		return true;
	}
	
	private function endOfTimeToStartGameEval( param: Object  ): Boolean
	{
		if ( this.newGameFlag )
		{
		
			var it: IIterator= this.entsCollection.iterator(  );
			
			while( it.hasNext(  ) )
			{
				var theDrop = it.next( ).secondVal;
			
				theDrop.destroy( );			
			}		
			
			this.numberDelta*= DELTA_COEF;		
		}
		
		return this.newGameFlag;
	}
	
	private function endOfTimeToVictoryEval( param: Object ): Boolean
	{
		if ( this.endOfGameFlag )
		{
		
			var it: IIterator= this.entsCollection.iterator(  );
			
			while( it.hasNext(  ) )
			{
				var bubble = it.next( ).secondVal;
			
				bubble.destroy( );			
			}				
		}	
		
		return this.endOfGameFlag;
	}
	
	private function overDropsToDefeatEval( param: Object ): Boolean
	{
		return true;
	}
	
	private function defeatToEndOfGameEval( param: Object ): Boolean
	{
		return true;	
	}
	
	private function victoryToEndOfGameEval( param: Object ): Boolean
	{
		return true;
	}
	
	private function onDropClicked( param: Object )
	{
		var theClickedId: String = param.dropId;
		
		var theClickedDrop: Drop = Drop( this.entsCollection.getItem( Object( theClickedId ) ) );
		
		theClickedDrop.destroy( );
	}
	
	public function gameCanRun( )
	{
		this.feedbackClickedFlag = true;
	}
	
	public function newGame( )
	{
		this.newGameFlag = true;
	}
	
	public function endGame( )
	{
		this.endOfGameFlag = true;
	}
}
